use std::sync::Arc;

use axum::Router;
use sqlx::PgPool;
use tokio::{net::TcpListener, sync::Mutex};

use crate::routes::member::migrate::MigrationStore;
use crate::routes::routes;
use crate::AppState;

pub async fn serve(pool: PgPool) {
    let migration_store = Arc::new(Mutex::new(MigrationStore::default()));

    let app_state = AppState {
        pool,
        migration_store,
    };

    // Serve app
    let app = Router::new().nest("/v1", routes()).with_state(app_state);

    let listener = TcpListener::bind("127.0.0.1:3000")
        .await
        .expect("Error while initializing listener");

    tracing::info!("Listening on {}", listener.local_addr().unwrap());

    axum::serve(listener, app)
        .await
        .expect("Error while serving axum application");
}
